#include <stdio.h>
#include <stdlib.h>
#include <pwd.h>
#include <sys/types.h>
#include <rsbac/types.h>
#include <rsbac/aci_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include "nls.h"

int main(int argc, char ** argv)
{
  int attr_list[RSBAC_USER_NR_ATTRIBUTES] = RSBAC_USER_ATTR_LIST;
  int res = 0;
  char tmp1[80],tmp2[120];
  int j;
  union rsbac_attribute_value_t value;
  union rsbac_target_id_t tid;
  enum rsbac_attribute_t attr;
  struct passwd * user_info_p;
  u_int position, catval;
  __u64 k;
  char * progname;
  int verbose = 0;
  int printall = 0;
  int add = 0;
  int remove = 0;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);

  progname = argv[0];
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'v':
                verbose++;
                break;
              case 'p':
                printall=1;
                break;
              case 'a':
                add=1;
                break;
              case 'm':
                remove=1;
                break;
              case 'n':
                {
                  char tmp[80];
                  int i;

                  for(i=0; i<R_NONE; i++)
                    printf("%s\n", get_request_name(tmp, i));
                  exit(0);
                }
              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  if(   (argc>2)
     && !strcmp(argv[2],"log_user_based")
    )
    {
      char * progname = argv[0];
      boolean rused = FALSE;
      boolean wused = FALSE;
      enum rsbac_adf_request_t request;
      rsbac_request_vector_t request_vector = 0;

      value.log_user_based = 0;
      if(   (!(user_info_p = getpwnam(argv[1])))
         && (   (!(user_info_p = getpwuid(strtol(argv[1],0,10))))
             || (!(user_info_p->pw_uid) && strcmp("0",argv[1])) )
        )
        {
          fprintf(stderr, gettext("%s: Invalid user %s!\n\n"), argv[0], argv[1]);
          exit(1);
        }
      tid.user = user_info_p->pw_uid;

      argv+=2;
      argc-=2;

      if(add || remove)
        {
          res = rsbac_get_attr(T_USER, &tid, A_log_user_based, &value, FALSE);
          error_exit(res);
        }
      while(argc > 1)
        {
          if(strlen(argv[1]) == R_NONE)
            {
              int j;
              rsbac_request_vector_t tmp_rv;

              for(j=0; j<R_NONE; j++)
                if(   (argv[1][j] != '0')
                   && (argv[1][j] != '1')
                  )
                  {
                    fprintf(stderr, gettext("%s: Invalid mask vector %s\n"), progname, argv[1]);
                    exit(1);
                  }
              strtou64acl(argv[1], &tmp_rv);
              request_vector |= tmp_rv;
              argv++;
              argc--;
              continue;
            }
          request = get_request_nr(argv[1]);
          if(request == R_NONE)
            {
              request = strtol(argv[1],0,10);
              if(   (request >= R_NONE)
                 || (   (request == 0)
                     && strcmp(argv[1],"0")
                    )
                )
                {
                  if(!strcmp(argv[1],"RW"))
                    {
                      request_vector |= RSBAC_READ_WRITE_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"SY"))
                    {
                      request_vector |= RSBAC_SYSTEM_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"SE"))
                    {
                      request_vector |= RSBAC_SECURITY_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"R"))
                    {
                      request_vector |= RSBAC_READ_REQUEST_VECTOR;
                      rused = TRUE;
                    }
                  else
                  if(!strcmp(argv[1],"W"))
                    {
                      request_vector |= RSBAC_WRITE_REQUEST_VECTOR;
                      wused = TRUE;
                    }
                  else
                  if(!strcmp(argv[1],"A"))
                    {
                      request_vector |= RSBAC_ALL_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"UA"))
                    {
                      request_vector = 0;
                    }
                  else
                  if(!strcmp(argv[1],"NWS"))
                    {
                      request_vector |= RSBAC_NWS_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWR"))
                    {
                      request_vector |= RSBAC_NWR_REQUEST_VECTOR;
                      rused = TRUE;
                    }
                  else
                  if(!strcmp(argv[1],"NWW"))
                    {
                      request_vector |= RSBAC_NWW_REQUEST_VECTOR;
                      wused = TRUE;
                    }
                  else
                  if(!strcmp(argv[1],"NWC"))
                    {
                      request_vector |= RSBAC_NWC_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWE"))
                    {
                      request_vector |= RSBAC_NWE_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWA"))
                    {
                      request_vector |= RSBAC_NWA_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWF"))
                    {
                      request_vector |= RSBAC_NWF_REQUEST_VECTOR;
                    }
                  else
                  if(!strcmp(argv[1],"NWM"))
                    {
                      request_vector |= RSBAC_NWM_REQUEST_VECTOR;
                    }
                  else
                    { /* end of requests */
                      break;
                    }
                }
            }
          else
            {
              request_vector |= ((rsbac_request_vector_t) 1 << request);
            }
          argv++;
          argc--;
        }
      if(rused && wused)
        {
          request_vector |= RSBAC_READ_WRITE_OPEN_REQUEST_VECTOR;
        }
      if(remove)
          value.log_user_based &= ~request_vector;
      else
          value.log_user_based |= request_vector;

      if(printall)
        {
          int i;

          for (i=0; i<R_NONE; i++)
            if(value.log_user_based & ((rsbac_request_vector_t) 1 << i))
              printf(" %s\n", get_request_name(tmp1,i));
        }

      res = rsbac_set_attr(T_USER, &tid, A_log_user_based, &value);
      error_exit(res);
      exit(0);
    }

/* normal attributes */
  switch(argc)
    {
      case 1:
      case 2:
      case 3:
        printf(gettext("%s (RSBAC %s)\n***\n"), argv[0], VERSION);
        printf(gettext("Use: %s user attribute [position] value\n\n"), argv[0]);  
        printf(gettext("Use: %s [switches] user log_user_based [request-list]\n\n"), argv[0]);  
        printf(gettext("-p = print resulting requests, -a = add, not set, -m = remove, not set\n"));
        printf(gettext("- attribute (string) and values = see following list:\n"));
        printf(gettext("mac_categories\t\t(with additional parameter position)\n\t\t\t0=no, 1=yes\n"));
        printf(gettext("log_user_based\t\t(with space separated list of requests)\n\t\t\t0=no, 1=yes\n"));
        for (j=0;j<RSBAC_USER_NR_ATTRIBUTES;j++)
          {
            get_attribute_name(tmp1,attr_list[j]);
            get_attribute_param(tmp2,attr_list[j]);
            printf("%s \t%s\n",tmp1,tmp2);
          }
        break;

      case 4:
        attr = get_attribute_nr(argv[2]);
        if(attr == A_none)
          {
            fprintf(stderr, gettext("%s: Invalid Attribute %s!\n"), argv[0], argv[2]);
            exit(1);
          }
        if(   (!(user_info_p = getpwnam(argv[1])))
           && (   (!(user_info_p = getpwuid(strtol(argv[1],0,10))))
               || (!(user_info_p->pw_uid) && strcmp("0",argv[1])) )
          )
            {
              fprintf(stderr, gettext("%s: Invalid user %s!\n\n"), argv[0], argv[1]);
              exit(1);
            }
        tid.user = user_info_p->pw_uid;

        switch(attr)
          {
            case A_none:
              fprintf(stderr, gettext("%s: Invalid attribute %s\n"), argv[0], argv[3]);
              exit(1);
            case A_mac_categories:
              if(strlen(argv[3]) != RSBAC_MAC_NR_CATS)
                {
                  fprintf(stderr, gettext("%s: Invalid attribute value, length must be %i\n"), argv[0],
                          RSBAC_MAC_NR_CATS);
                  exit(1);
                }

            default:
              value.dummy = strtol(argv[3],0,10);
          }
        res = rsbac_set_attr(T_USER, &tid, attr, &value);
        error_exit(res);
        exit(0);

      case 5:
        if(!strcmp(argv[2],"mac_categories"))
          {
            if(   (!(user_info_p = getpwnam(argv[1])))
               && (   (!(user_info_p = getpwuid(strtol(argv[1],0,10))))
                   || (!(user_info_p->pw_uid) && strcmp("0",argv[1])) )
              )
              {
                fprintf(stderr, gettext("%s: Invalid user %s!\n\n"), argv[0], argv[1]);
                exit(1);
              }
            tid.user = user_info_p->pw_uid;
            position = strtol(argv[3],0,10);
            if(position > RSBAC_MAC_MAX_CAT)
              {
                fprintf(stderr, gettext("Invalid position counter %s\n"), argv[4]);
                exit(1);
              }
            catval = strtol(argv[4],0,10);
            if(catval > 1)
              {
                fprintf(stderr, gettext("Invalid value %s\n"), argv[4]);
                exit(1);
              }
            res = rsbac_get_attr(T_USER, &tid, A_mac_categories, &value, FALSE);
            error_exit(res);
            k = ((__u64) 1) << position;
            if(catval)
              value.mac_categories |= k;
            else
              value.mac_categories &= ~k;
            res = rsbac_set_attr(T_USER, &tid, A_mac_categories, &value);
            error_exit(res);
            exit(0);
          }

      default:
    }
  exit(1);
}
